#!/usr/bin/env python3
import re
from urllib.request import urlopen
from typing import Tuple

from deemix.itemgen import (
    generateTrackItem,
    generateAlbumItem,
    generatePlaylistItem,
    generateArtistItem,
    generateArtistDiscographyItem,
    generateArtistTopItem,
)
from deemix.errors import LinkNotRecognized, LinkNotSupported

__version__ = "3.6.6"


# Returns the Resolved URL, the Type and the ID
def parseLink(link: str) -> Tuple[str, str | None, str | None]:
    if "deezer.page.link" in link:
        link = urlopen(link).url  # Resolve URL shortner
    # Remove extra stuff
    if "?" in link:
        link = link[: link.find("?")]
    if "&" in link:
        link = link[: link.find("&")]
    if link.endswith("/"):
        link = link[:-1]  #  Remove last slash if present

    if "deezer" not in link:
        return link, None, None  # return if not a deezer link

    type_patterns = {
        "track": r"/track/(.+)",
        "playlist": r"/playlist/(\d+)",
        "album": r"/album/(.+)",
        "artist_top": r"/artist/(\d+)/top_track",
        "artist_discography": r"/artist/(\d+)/discography",
        "artist": r"/artist/(\d+)",
    }

    for t, p in type_patterns.items():
        if m := re.search(p, link):
            link_type, link_id = t, m.group(1)
            return link, link_type, link_id

    return link, None, None


def generateDownloadObject(dz, link, bitrate, listener=None):
    link, link_type, link_id = parseLink(link)

    if not (link_type and link_id):
        raise LinkNotRecognized(link)

    if link_type == "track":
        return generateTrackItem(dz, link_id, bitrate)
    if link_type == "album":
        return generateAlbumItem(dz, link_id, bitrate)
    if link_type == "playlist":
        return generatePlaylistItem(dz, link_id, bitrate)
    if link_type == "artist":
        return generateArtistItem(dz, link_id, bitrate, listener)
    if link_type == "artist_discography":
        return generateArtistDiscographyItem(dz, link_id, bitrate, listener)
    if link_type == "artist_top":
        return generateArtistTopItem(dz, link_id, bitrate)

    raise LinkNotSupported(link)
